@@ -23,26 +23,43 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-`possibleNumeric` <- function(x) {
-    if (all(is.na(x))) {
+`possibleNumeric` <- function(x, each = FALSE) {
+    result <- rep(NA, length(x))
+    isna <- is.na(x)
+    if (all(isna)) {
+        if (each) {
+            return(result)
+        }
         return(FALSE)
     }
     if (inherits(x, "haven_labelled") || inherits(x, "declared")) {
-        num <- Recall(unclass(x))
-        labels <- attr(x, "labels")
-        if (!is.null(labels) && num) {
+        num <- Recall(unclass(x), each = each)
+        labels <- attr(x, "labels", exact = TRUE)
+        if (!is.null(labels) && !each && num) {
             return(Recall(labels))
         }
         return(num)
     }
     if (is.numeric(x)) {
+        if (each) {
+            result[!isna] <- TRUE
+            return(result)
+        }
         return(TRUE)
     }
     if (is.factor(x)) {
-        return(!any(is.na(suppressWarnings(as.numeric(levels(x))))))
+        x <- as.character(x)
     }
-    if (any(grepl("[^!-~ ]", x))) {
-        return(FALSE)
+    multibyte <- grepl("[^!-~ ]", x)
+    if (any(multibyte)) {
+        isna[multibyte] <- TRUE
+        result[multibyte] <- FALSE
+        x[multibyte] <- NA
+    }
+    if (each) {
+        x <- suppressWarnings(as.numeric(na.omit(x)))
+        result[!isna] <- !is.na(x)
+        return(result)
     }
     return(!any(is.na(suppressWarnings(as.numeric(na.omit(x))))))
 }
