@@ -36,7 +36,7 @@
                 xs <- unlist(strsplit(x, split = arrows[j]))
                 if (length(xs) == 2) {
                     if (all(grepl("\\*|\\+", xs))) {
-                        stopError("The outcome should be one condition (only).")
+                        stopError("The outcome should be a single condition.")
                     }
                     if (j < 3) { 
                         if (grepl("\\*|\\+", xs[2])) {
