@@ -79,7 +79,9 @@
             }
         }
     }
-    scollapse <- scollapse | any(nchar(colnames(implicants)) > 1) | any(grepl(mvregexp, unlist(sols$solution))) 
+    scollapse <- scollapse |
+                any(nchar(colnames(implicants)) > 1) |
+                any(grepl(mvregexp, unlist(sols$solution)))
     expression <- unlist(lapply(sols$solution, function(x) {
         if (!scollapse) x <- gsub("\\*", "", x)
         return(paste(x, collapse = " + "))  
